// lista6.h - Genericka klasa listi.

#ifndef _lista6_h_
#define _lista6_h_

#include <iostream>
using namespace std;

class GNemaTek {};    // KLASA GRESAKA: Nema tekuceg elementa.
inline ostream& operator<< (ostream& d, const GNemaTek&)
  { return d << "*** Nema tekuceg elementa!"; }

template <typename T> // GENERICKA KLASA LISTI:
  class Lista {

    struct Elem {                    // Element liste:
      T pod;                         // - sadrzani podatak,
      Elem* sled;                    // - sledeci element,
      Elem (const T& p, Elem* s=0)   // - konstruktor.
        { pod = p; sled = s; }
    };
    Elem *prvi, *posl;               // Prvi i poslednji element.
    mutable Elem *tek, *pret;        // Tekuci i prethodni element.
    void kopiraj (const Lista& lst); // Kopiranje u objekat.
  public:
    Lista ()                         // Stvaranje prazne liste.
      { prvi = posl = tek = pret = 0; }
    Lista (const Lista& lst) { kopiraj (lst); } // Konstruktor kopije.
    ~Lista () { isprazni (); }                  // Destruktor.
    Lista& operator= (const Lista& lst) {       // Dodela vrednosti.
      if (this != &lst) { isprazni (); kopiraj (lst); }
      return *this;
    }
    Lista& dodaj (const T& t) {                 // Dodavanje na kraj.
      posl = (!prvi ? prvi : posl->sled) = new Elem (t);
      return *this;
    }
    Lista& naPrvi ()                 // Postavljanje na prvi element
      { tek = prvi; pret = 0; return *this; }   // - promenljive liste,
    const Lista& naPrvi () const                  
      { tek = prvi; pret = 0; return *this; }   // - nepromenljive liste.
    Lista& naSled () {               // Pomeranje na sledeci element
      pret = tek;                    // - promenljive liste,
      if (tek) tek = tek->sled;
      return *this;
    }
    const Lista& naSled () const {   // - nepromenljve liste.
      pret = tek;
      if (tek) tek = tek->sled;
      return *this;
    }
    bool imaTek () const { return tek != 0; }   // Ima li tekuceg?
    T& uzmiTek () {                  // Pristup tekucem podatku
      if (! tek) throw GNemaTek ();  // - promenljive liste,
      return tek->pod;
    }
    const T& uzmiTek () const {      // - nepromenljive liste.
      if (! tek) throw GNemaTek ();
      return tek->pod;
    }
    Lista& izbaciTek () {            // Izbacivanje tekuceg elementa.
      if (! tek) throw GNemaTek ();
      Elem* stari = tek;
      tek = tek->sled;
      (!pret ? prvi : posl->sled) = tek;
      if (! tek) posl = pret;
      return *this;
    }
    Lista& isprazni ();              // Praznjenje liste.
  }; // class Lista<T>

template <typename T> // DEFINICIJE METODA KLASE LISTI:
  void Lista<T>::kopiraj (const Lista& lst) {   // Kopiranje u objekat.
    prvi = posl = tek = pret = 0;
    for (Elem* pok=lst.prvi; pok; pok=pok->sled) {
      Elem* novi = new Elem (pok->pod);
      posl = (!prvi ? prvi : posl->sled) = novi;
      if (pok == lst.pret) pret = pok;
      if (pok == lst.tek) tek = pok;
    }
  }

template <typename T>
  Lista<T>& Lista<T>::isprazni () {             // Praznjenje liste.
    while (prvi) { Elem* stari = prvi; prvi = prvi->sled; delete stari; }
    posl = tek = pret = 0;
    return *this;
  }

#endif
